window.Gleam.initSearch([{"doc":"dinostore","title":"dinostore","content":"# dinostore\n\nBindings to the [`Deno.Kv`](https://docs.deno.com/deploy/kv/manual/) database.\n\n[![Package Version](https://img.shields.io/hexpm/v/dinostore)](https://hex.pm/packages/dinostore)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/dinostore/)\n\n## Setup\n\n```sh\ngleam add dinostore\n\n# You will also need gleam_javascript if not already installed\ngleam add gleam_javascript\n```\n\nYour `gleam.toml` config will need the JavaScript runtime set to `deno`.\n\n```toml\ntarget = &quot;javascript&quot;\n\n[javascript]\nruntime = &quot;deno&quot;\n```\n\nMake sure that your `deno.json` also contains the following configuration:\n\n```json\n{\n  &quot;unstable&quot;: [&quot;kv&quot;]\n}\n```\n\n## A note on serialization\n\nDeno KV uses the [structured clone algorithm](https://developer.mozilla.org/en-US/docs/Web/API/Web_Workers_API/Structured_clone_algorithm) to store values. This means that Gleam values such as custom types will be converted to JavaScript objects before being stored in the database. For example, consider the following custom type:\n\n```gleam\ntype Book {\n  Book(title: String, author: String)\n}\n```\n\nA constructed value such as `Book(title: &quot;foo&quot;, author: &quot;bar&quot;)` will be stored in the database as a JavaScript object when cloned:\n\n```json\n{ &quot;title&quot;: &quot;foo&quot;, &quot;author&quot;: &quot;bar&quot; }\n```\n\n`Dynamic` decoders should be carefully written with this in mind. It may be helpful to inspect the value retrieved from the database before decoding it to see how it has changed.\n\nFor convenience, `List`s are automatically converted to JavaScript arrays before being stored in the database.\n\n## Example\n\nIn the following program, several `let assert`s are used when decoding `Dynamic` data. When creating wrapper functions for the database, you can be certain that the retrieved data is correctly structured, provided that these wrapper functions are consistently used instead of directly accessing the raw database. Ultimately, it&#39;s up to you to decide how to handle the case where parsing data fails.\n\n```gleam\nimport dinostore/atomic\nimport dinostore/key\nimport dinostore/kv\nimport gleam/dynamic\nimport gleam/io\nimport gleam/javascript/promise.{type Promise}\nimport gleam/option.{None}\n\ntype Cat =\n  #(String, Int)\n\nfn decode_cat(x: dynamic.Dynamic) {\n  dynamic.tuple2(dynamic.string, dynamic.int)(x)\n}\n\nfn add_cat(conn: kv.Connection, cat: Cat) -&gt; Promise(Result(String, Nil)) {\n  let primary_key = [key.s(&quot;cats&quot;), key.ulid()]\n  let by_name = [key.s(&quot;cats_by_name&quot;), key.s(cat.0)]\n\n  atomic.new(conn)\n  // Check that the key does not already exist\n  |&gt; atomic.check(by_name, None)\n  // Create the primary and secondary keys\n  |&gt; atomic.set(primary_key, cat, expiration: None)\n  |&gt; atomic.set(by_name, key.unwrap(primary_key), expiration: None)\n  // Attempt to commit the mutations\n  |&gt; atomic.commit\n}\n\nfn get_cat_by_name(\n  conn: kv.Connection,\n  name: String,\n) -&gt; Promise(Result(Cat, kv.GetError)) {\n  let secondary_key = [key.s(&quot;cats_by_name&quot;), key.s(name)]\n\n  use primary_key_entry &lt;- promise.try_await(kv.get(conn, secondary_key))\n  let assert Ok(primary_key) = key.decode(primary_key_entry.value)\n\n  use entry &lt;- promise.try_await(kv.get(conn, primary_key))\n  let assert Ok(cat) = decode_cat(entry.value)\n\n  promise.resolve(Ok(cat))\n}\n\npub fn main() {\n  use conn &lt;- promise.await(kv.open_with_path(&quot;:memory:&quot;))\n\n  // Add two cats to the database. Note that here we are ignoring add_cat&#39;s\n  // results; in a more sophisticated program you would want to handle error\n  // cases appropriately.\n  use _ &lt;- promise.await(add_cat(conn, #(&quot;Fluffy&quot;, 2)))\n  use _ &lt;- promise.await(add_cat(conn, #(&quot;Oreo&quot;, 10)))\n\n  // Retrieve a cat by its name\n  use cat &lt;- promise.await(get_cat_by_name(conn, &quot;Oreo&quot;))\n\n  let _ = io.debug(cat)\n  //=&gt; #(&quot;Oreo&quot;, 10)\n\n  promise.resolve(Nil)\n}\n```\n\nA more involved example can be found in [./test](https://github.com/MystPi/dinostore/tree/main/test).\n\n## Documentation\n\nFurther documentation can be found at &lt;https://hexdocs.pm/dinostore&gt;.\n\n## Development\n\n```sh\ngleam run   # Run the project\ngleam test  # Run the test project\n```\n","url":"index.html"},{"doc":"dinostore/ulid","title":"new","content":"pub fn new() -&gt; String\n Generate a new ULID.\n\n [Official Documentation](https://jsr.io/@std/ulid)\n\nSynonyms:\ndinostore/ulid.new\ndinostore/ulid new","url":"dinostore/ulid.html#new"},{"doc":"dinostore/ulid","title":"dinostore/ulid","content":" Read more about ULIDs in the\n [official Deno docs](https://docs.deno.com/deploy/kv/manual/key_space/#universally-unique-lexicographically-sortable-identifiers-(ulids)).\n","url":"dinostore/ulid.html"},{"doc":"dinostore/key","title":"JsKey","content":"pub type JsKey\n\n\nSynonyms:\ndinostore/key.JsKey\ndinostore/key JsKey","url":"dinostore/key.html#JsKey"},{"doc":"dinostore/key","title":"Key","content":"pub type Key =\n  List(KeyPart)\n A key is simply a list of `KeyPart`s.\n\nSynonyms:\ndinostore/key.Key\ndinostore/key Key","url":"dinostore/key.html#Key"},{"doc":"dinostore/key","title":"KeyPart","content":"pub type KeyPart {\n  StringPart(value: String)\n  NumberPart(value: Float)\n  BoolPart(value: Bool)\n}\n\nStringPart(value: String)\n\n\nNumberPart(value: Float)\n Number parts are represented as floats since JavaScript does not differentiate\n between integers and floats.\n\nBoolPart(value: Bool)\n\n\nSynonyms:\ndinostore/key.KeyPart\ndinostore/key KeyPart","url":"dinostore/key.html#KeyPart"},{"doc":"dinostore/key","title":"b","content":"pub fn b(b: Bool) -&gt; KeyPart\n Helper function to create a `BoolPart`.\nSynonyms:\ndinostore/key.b\ndinostore/key b","url":"dinostore/key.html#b"},{"doc":"dinostore/key","title":"decode","content":"pub fn decode(\n  x: Dynamic,\n) -&gt; Result(List(KeyPart), List(DecodeError))\n Decode a normal JavaScript key (e.g. one created with `unwrap` and stored in\n the database) into a Gleam `Key`.\n\n Both this function and `unwrap` are helpful for creating\n [secondary indexes](https://docs.deno.com/deploy/kv/manual/#improve-querying-with-secondary-indexes)\n where a primary key is stored as a secondary index&#39;s value.\n\nSynonyms:\ndinostore/key.decode\ndinostore/key decode","url":"dinostore/key.html#decode"},{"doc":"dinostore/key","title":"n","content":"pub fn n(n: Float) -&gt; KeyPart\n Helper function to create a `NumberPart`.\nSynonyms:\ndinostore/key.n\ndinostore/key n","url":"dinostore/key.html#n"},{"doc":"dinostore/key","title":"s","content":"pub fn s(s: String) -&gt; KeyPart\n Helper function to create a `StringPart`.\nSynonyms:\ndinostore/key.s\ndinostore/key s","url":"dinostore/key.html#s"},{"doc":"dinostore/key","title":"ulid","content":"pub fn ulid() -&gt; KeyPart\n Create a ULID key part. To create a new ULID without a key, use the\n `new` function in the `dinostore/ulid` module.\n\n [Official Documentation](https://jsr.io/@std/ulid)\n\nSynonyms:\ndinostore/key.ulid\ndinostore/key ulid","url":"dinostore/key.html#ulid"},{"doc":"dinostore/key","title":"unwrap","content":"pub fn unwrap(key: List(KeyPart)) -&gt; JsKey\n Turn a Gleam `Key` into a normal JavaScript key (of the TS type\n `(string | number | boolean)[]`) by unwrapping the value of each `KeyPart`.\n Use `decode` to convert the key back to a `Key`.\n\n Both this function and `decode` are helpful for creating\n [secondary indexes](https://docs.deno.com/deploy/kv/manual/#improve-querying-with-secondary-indexes)\n where a primary key is stored as a secondary index&#39;s value.\n\n \n io.debug(unwrap([s(&quot;foo&quot;), n(1.0), b(True)]))\n // =&gt; #(&quot;foo&quot;, 1.0, True)\n // Gleam tuples are represented as JavaScript arrays under the hood\n \n\nSynonyms:\ndinostore/key.unwrap\ndinostore/key unwrap","url":"dinostore/key.html#unwrap"},{"doc":"dinostore/key","title":"dinostore/key","content":"","url":"dinostore/key.html"},{"doc":"dinostore/kv","title":"Connection","content":"pub type Connection\n\n\nSynonyms:\ndinostore/kv.Connection\ndinostore/kv Connection","url":"dinostore/kv.html#Connection"},{"doc":"dinostore/kv","title":"Entry","content":"pub type Entry {\n  Entry(key: Key, value: Dynamic, versionstamp: String)\n}\n An entry in the database. Since the value is `Dynamic`, it needs to be parsed\n before it can be used in any useful way.\n\nEntry(key: Key, value: Dynamic, versionstamp: String)\n\n\nSynonyms:\ndinostore/kv.Entry\ndinostore/kv Entry","url":"dinostore/kv.html#Entry"},{"doc":"dinostore/kv","title":"GetError","content":"pub type GetError {\n  NotFound(Key)\n}\n Errors that can be returned from functions that retrieve entries from the\n database.\n\nNotFound(Key)\n The key was not found in the database.\n\nSynonyms:\ndinostore/kv.GetError\ndinostore/kv GetError","url":"dinostore/kv.html#GetError"},{"doc":"dinostore/kv","title":"close","content":"pub fn close(conn: Connection) -&gt; Nil\n Use this function to close a database connection manually.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.Kv#method_close_0)\n\nSynonyms:\ndinostore/kv.close\ndinostore/kv close","url":"dinostore/kv.html#close"},{"doc":"dinostore/kv","title":"delete","content":"pub fn delete(\n  conn: Connection,\n  key key: List(KeyPart),\n) -&gt; Promise(Nil)\n Delete the value for the given key from the database. If no value exists for\n the key, this operation is a no-op (hence the `Nil` return).\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.Kv#method_delete_0)\n\nSynonyms:\ndinostore/kv.delete\ndinostore/kv delete","url":"dinostore/kv.html#delete"},{"doc":"dinostore/kv","title":"get","content":"pub fn get(\n  conn: Connection,\n  key key: List(KeyPart),\n) -&gt; Promise(Result(Entry, GetError))\n Retrieve the value and versionstamp for the given key from the database,\n returning an [`Entry`](#Entry) or [`GetError`](#GetError).\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.Kv#method_get_0)\n\nSynonyms:\ndinostore/kv.get\ndinostore/kv get","url":"dinostore/kv.html#get"},{"doc":"dinostore/kv","title":"list","content":"pub fn list(\n  conn: Connection,\n  prefix prefix: List(KeyPart),\n  reverse reverse: Bool,\n  limit limit: Option(Int),\n  cursor cursor: Option(String),\n) -&gt; Promise(List(Entry))\n Retrieve all entries with the given key prefix from the database. The entries\n will be returned in reverse lexicographical order if `reverse` is `True`.\n `cursor` can be used to start the retrieval from a specific point.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.Kv#method_list_0)\n\n [Read more about key ordering](https://docs.deno.com/deploy/kv/manual/key_space/#key-part-ordering)\n\n \n use entries &lt;- promise.await(list([&quot;books&quot;], reverse: False, limit: None))\n use entry &lt;- list.each(entries)\n io.debug(entry)\n \n\n\nSynonyms:\ndinostore/kv.list\ndinostore/kv list","url":"dinostore/kv.html#list"},{"doc":"dinostore/kv","title":"open","content":"pub fn open() -&gt; Promise(Connection)\n Open a new `Deno.Kv` connection to the default database location. This may be\n local or remote, depending on where the program is run.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.openKv)\n\n \n use conn &lt;- promise.await(open())\n \n\nSynonyms:\ndinostore/kv.open\ndinostore/kv open","url":"dinostore/kv.html#open"},{"doc":"dinostore/kv","title":"open_with_path","content":"pub fn open_with_path(path: String) -&gt; Promise(Connection)\n Open a new `Deno.Kv` connection to the database at the given path. The path\n `:memory:` can be used to connect to an in-memory database.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.openKv)\n\n \n use conn &lt;- promise.await(open_with_path(&quot;:memory:&quot;))\n \n\nSynonyms:\ndinostore/kv.open_with_path\ndinostore/kv open_with_path","url":"dinostore/kv.html#open_with_path"},{"doc":"dinostore/kv","title":"set","content":"pub fn set(\n  conn: Connection,\n  key key: List(KeyPart),\n  value value: a,\n  expiration expiration: Option(Int),\n) -&gt; Promise(String)\n Set the value for the given key in the database. If a value already exists\n for the key, it will be overwritten. This operation, by definition, cannot\n fail, so it returns the new versionstamp of the key.\n\n An expiration (time-to-live, or TTL) can be specified in milliseconds. The\n key will be deleted from the database at earliest after the specified number\n of milliseconds have elapsed. Once the duration has passed, the key may still\n be visible or some additional time. If an expiration is not specified, the\n key will not expire.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.Kv#method_set_0)\n\nSynonyms:\ndinostore/kv.set\ndinostore/kv set","url":"dinostore/kv.html#set"},{"doc":"dinostore/kv","title":"dinostore/kv","content":" Bindings to the [`Deno.Kv`](https://docs.deno.com/deploy/kv/manual/) database.\n","url":"dinostore/kv.html"},{"doc":"dinostore/atomic","title":"AtomicOperation","content":"pub type AtomicOperation\n\n\nSynonyms:\ndinostore/atomic.AtomicOperation\ndinostore/atomic AtomicOperation","url":"dinostore/atomic.html#AtomicOperation"},{"doc":"dinostore/atomic","title":"check","content":"pub fn check(\n  op: AtomicOperation,\n  key key: List(KeyPart),\n  versionstamp versionstamp: Option(String),\n) -&gt; AtomicOperation\n Add a key-versionstamp check to the operation. If the versionstamp is `None`,\n the check will make sure the key-value pair does not already exist in the\n database.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation#method_check_0)\n\nSynonyms:\ndinostore/atomic.check\ndinostore/atomic check","url":"dinostore/atomic.html#check"},{"doc":"dinostore/atomic","title":"check_entry","content":"pub fn check_entry(\n  op: AtomicOperation,\n  entry: Entry,\n) -&gt; AtomicOperation\n Add a check based on an `Entry`&#39;s key and versionstamp to the operation.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation#method_check_0)\n\nSynonyms:\ndinostore/atomic.check_entry\ndinostore/atomic check_entry","url":"dinostore/atomic.html#check_entry"},{"doc":"dinostore/atomic","title":"commit","content":"pub fn commit(\n  op: AtomicOperation,\n) -&gt; Promise(Result(String, Nil))\n Commit the operation to the database. A return of `Ok(versionstamp)` means\n that the operation was successful and all checks passed; a return of\n `Error(Nil)` means that at least one check failed and no mutations were\n made.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation#method_commit_0)\n\nSynonyms:\ndinostore/atomic.commit\ndinostore/atomic commit","url":"dinostore/atomic.html#commit"},{"doc":"dinostore/atomic","title":"delete","content":"pub fn delete(\n  op: AtomicOperation,\n  key: List(KeyPart),\n) -&gt; AtomicOperation\n Add to the operation a mutation that deletes the specified key if all checks\n pass during the commit.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation#method_delete_0)\n\nSynonyms:\ndinostore/atomic.delete\ndinostore/atomic delete","url":"dinostore/atomic.html#delete"},{"doc":"dinostore/atomic","title":"new","content":"pub fn new(conn: Connection) -&gt; AtomicOperation\n Create a new [`AtomicOperation`](#AtomicOperation).\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation)\n\nSynonyms:\ndinostore/atomic.new\ndinostore/atomic new","url":"dinostore/atomic.html#new"},{"doc":"dinostore/atomic","title":"set","content":"pub fn set(\n  op: AtomicOperation,\n  key key: List(KeyPart),\n  value value: a,\n  expiration expiration: Option(Int),\n) -&gt; AtomicOperation\n Add to the operation a mutation that sets the value of the specified key to\n the specified value if all checks pass during the commit.\n\n An expiration (time-to-live, or TTL) can be specified in milliseconds. The\n key will be deleted from the database at earliest after the specified number\n of milliseconds have elapsed. Once the duration has passed, the key may still\n be visible or some additional time. If an expiration is not specified, the\n key will not expire.\n\n [Official Documentation](https://docs.deno.com/api/deno/~/Deno.AtomicOperation#method_set_0)\n\nSynonyms:\ndinostore/atomic.set\ndinostore/atomic set","url":"dinostore/atomic.html#set"},{"doc":"dinostore/atomic","title":"dinostore/atomic","content":" This module provides bindings to the [`Deno.AtomicOperation`](https://docs.deno.com/api/deno/~/Deno.AtomicOperation)\n API. Read more about it [here](https://docs.deno.com/deploy/kv/manual/transactions/).\n","url":"dinostore/atomic.html"}]);